@@ -74,16 +74,24 @@ main(argc,argv)
 			    case '3':
 				ciph=CFS_THREE_DES;
 				break;
+#ifdef NOTDEF				
+			    case 'b':
+				ciph=CFS_BLOWFISH;
+				break;
+#endif				
 			    case 'm':
 				ciph=CFS_MACGUFFIN;
 				break;
+			    case 's':
+				ciph=CFS_SAFER_SK128;
+				break;
 			    default:
-				fprintf(stderr,"usage: ccat [-3m] file ...\n");
+				fprintf(stderr,"usage: ccat [-3bms] file...\n");
 				exit(1);
 			}
 	}
 	if (argc<1) {
-		fprintf(stderr,"Usage: ccat [-3m] file ...\n");
+		fprintf(stderr,"Usage: ccat [-3bms] file...\n");
 		exit(1);
 	}
 	if ((pw=getpassword("Key:"))==NULL) {
