@@ -299,17 +299,18 @@ doencrypt(k,s,l,salt,vect)
      int salt;
      char *vect;
 {
-	int i,j;
+	int i,j,smsize;
 	
+	smsize=k->smsize;
 	for (i=0; i<l; i+=8) {
 		for (j=0; j<8; j++)
-			s[i+j] ^= k->primask[(i+j+salt)%SMSIZE]
+			s[i+j] ^= k->primask[(i+j+salt)%smsize]
 				^ vect[j]
-				^ (((i+j+salt)/SMSIZE)&0377);
+				^ (((i+j+salt)/smsize)&0377);
 				/* makes big offsets have different masks */
 		cipher(k,&s[i],0);
 		for (j=0; j<8; j++)
-			s[i+j] ^= k->secmask[(i+j+salt)%SMSIZE];
+			s[i+j] ^= k->secmask[(i+j+salt)%smsize];
 	}
 }
 
@@ -320,16 +321,17 @@ dodecrypt(k,s,l,salt,vect)
      int salt;
      char *vect;
 {
-	int i,j;
+	int i,j,smsize;
 	
+	smsize=k->smsize;
 	for (i=0; i<l; i+=8) {
 		for (j=0; j<8; j++)
-			s[i+j] ^= k->secmask[(i+j+salt)%SMSIZE];
+			s[i+j] ^= k->secmask[(i+j+salt)%smsize];
 		cipher(k,&s[i],1);
 		for (j=0; j<8; j++)
-			s[i+j] ^= k->primask[(i+j+salt)%SMSIZE]
+			s[i+j] ^= k->primask[(i+j+salt)%smsize]
 				^ vect[j]
-				^ (((i+j+salt)/SMSIZE) & 0377);
+				^ (((i+j+salt)/smsize) & 0377);
 	}
 		
 }
@@ -547,8 +549,15 @@ fhtofd(f,mode)
 		close(curfd);	/* hope it was valid... */
 		openfd=NULL;
 	}
+
 	if (mode==0) { mode=CFS_WRITE; }
-	if ((fd=open(f->name,mode,0))<0) {
+	/* Phil Karn's hack for R/O file systems */
+	if ((fd=open(f->name,mode,0))<0 && errno == EROFS) {
+		mode = CFS_READ;        /* Force read and try again */
+		fd = open(f->name,mode,0);
+	}
+
+	if(fd<0) {
 		cfserrno=cfsno(errno);
 		if ((errno == EACCES) && (mode==CFS_WRITE)) { /* ugly hack */
 			int omode;
