@@ -1,8 +1,12 @@
 /*
  * ESM - Encrypted Session Manager
- * v1.0.0
+ * v1.0.1
  * matt blaze
- * july 1995
+ * October 1995
+ */
+
+/* SunOS 5 port by Greg Onufer, based in part on the pty_termios
+ * package written by Don Libes, NIST, 2/6/90
  */
 
 /*
@@ -76,6 +80,12 @@ char copyright2[] =
 #include <sys/signal.h>
 #include <unistd.h>
 #include <stdio.h>
+#ifdef SOLARIS2X
+#include <fcntl.h>
+#endif
+#ifdef PTMX
+#include <stropts.h>
+#endif
 #include "global.h"
 #include "rsaref.h"
 #include "esm.h"
@@ -96,7 +106,11 @@ struct	termios tt;
 struct	winsize win;
 int	lb;
 int	l;
+#ifdef PTMX
+char	*slave_name;
+#else
 char	line[] = "/dev/ptyXX";
+#endif
 int	aflg;
 
 #define REMOTE 0
@@ -173,8 +187,8 @@ main(argc, argv)
 		doshell(mode);
 	else {
 		/* main loop */
-		printf("ESM v1.0 - encrypted session manager\n");
-		printf("    by Matt Blaze, AT&T Bell Labs, June 1995\n");
+		printf("ESM v1.0.1 - encrypted session manager\n");
+		printf("    by Matt Blaze, AT&T Bell Labs, October 1995\n");
 		randinit();
 		cipherinit();
 		switch (mode) {
@@ -634,11 +648,11 @@ int ciphercalcin(ch)
 void
 finish()
 {
-	union wait status;
+	int status;
 	register int pid;
 	register int die = 0;
 
-	while ((pid = wait3((int *)&status, WNOHANG, 0)) > 0)
+	while ((pid = waitpid((pid_t)-1, &status, WNOHANG)) > 0)
 		if (pid == subchild)
 			die = 1;
 
@@ -879,6 +893,45 @@ done()
 	exit(0);
 }
 
+
+#ifdef PTMX
+
+getmaster()
+{
+	if ((master = open("/dev/ptmx", O_RDWR)) == -1)
+		return (-1);
+	if ((slave_name = (char *)ptsname(master)) == NULL ||
+		unlockpt(master) ||
+		grantpt(master)) {
+		close(master);
+		fail();
+	}
+	(void) ioctl(master, TIOCFLUSH, NULL);
+	(void) tcgetattr(0, &tt);
+    	(void) ioctl(0, TIOCGWINSZ, (char *)&win);
+}
+
+getslave()
+{
+	(void) setsid();
+	if ((slave = open(slave_name, O_RDWR)) < 0) {
+		fail();
+	}
+        if (ioctl(slave, I_PUSH, "ptem")) {
+		fail();
+        }
+        if (ioctl(slave, I_PUSH, "ldterm")) {
+		fail();
+        }
+        if (ioctl(slave, I_PUSH, "ttcompat")) {
+		fail();
+        }
+	(void) tcsetattr(slave, TCSAFLUSH, &tt);
+	(void) ioctl(slave, TIOCSWINSZ, (char *)&win);
+}
+
+#else
+
 getmaster()
 {
 	char *pty, *bank, *cp;
@@ -930,6 +983,8 @@ getslave()
 	(void) ioctl(slave, TIOCSCTTY, 0);
 }
 
+#endif
+
 waitenter()
 {
 	unsigned char b;
